Character = class("Character")

function Character:init(name,charFolder)

	self.name = name
	self.charFolder = charFolder
	
	if(charFolder ~= nil)then
		self.imageFolder = charFolder.."images/"
		self.voiceFolder = charFolder.."voice/"
	end
	
	self.descriptionImage = -1
	self.style = -1
	
	-- Not Implemented -- >> Use descriptionTextImage instead
	-- self.info1 = ""
	-- self.info2 = ""
	-- self.info3 = ""
	-- self.description = ""
	self.descriptionTextImageIndex = -1
	
	self.voiceList = {}
	self.styleList = {}
	self.descriptionImageList = {}
	self.descriptionTextImage = nil
	self.descriptionTextImageList = {}
end

function Character:setName(name)
	self.name = name
end

function Character:setVoiceFolder(path)
	self.voiceFolder = path
end

function Character:setStyle(num) --NUMBER
	self.style = num
end

function Character:getStyleImage(num) --NUMBER
	if(num == -1)then
		return nil
	end
	if(self.styleList[num] == nil)then
		path = self.imageFolder..num..".png"
		self.styleList[num] = love.graphics.newImage(path)
	end
	if(self.styleList[num]==nil)then
		error("LoveStory: Character Style Image "..num..".png not found!")
	end
	return self.styleList[num]
end

function Character:setDescriptionImage(num) --NUMBER
	self.descriptionImage = num
end

function Character:getDescriptionImage(num)
	if(num == -1)then
		return nil
	end
	if(self.descriptionImageList[num] == nil)then
		path = self.imageFolder.."big_"..num..".png"
		self.descriptionImageList[num] = love.graphics.newImage(path)
	end
	if(self.descriptionImageList[num]==nil)then
		error("LoveStory: Character Description "..num..".png Image not found!")
	end
	return self.descriptionImageList[num]
end

function Character:setDescriptionTextImage(num)
	self.descriptionTextImageIndex = num
end

function Character:clearTable(table)
	for k in pairs (table) do
		table [k] = nil
	end
end

function Character:clearCache()
	self:clearTable(self.voiceList)
	self:clearTable(self.styleList)
	self:clearTable(self.descriptionImageList)
	self:clearTable(self.descriptionTextImageList)
	self.descriptionTextImage = nil
	debug.text = "CACHE CLEAR"
end

function Character:getDescriptionTextImage() --standard = old way ... set self.desctiptionTextImageIndex for multiple
	--OLD WAY -> ONLY 1 IMAGE
	if(self.descriptionTextImageIndex == -1)then
		if(self.descriptionTextImage == nil)then
			path = self.imageFolder.."description_text.png"
			self.descriptionTextImage = love.graphics.newImage(path)
		end
		if(self.descriptionTextImage==nil)then
				error("LoveStory: Description Text Image not found! (OLD WAY)")
		end
		return self.descriptionTextImage
	--NEW WAY -> Multiple Images (see self.descriptionTextImageIndex)
	end
	if(self.descriptionTextImageIndex > -1)then
		local n = self.descriptionTextImageIndex
		if(self.descriptionTextImageList[n] == nil)then
			path = self.imageFolder.."description_text_"..n..".png"
			self.descriptionTextImageList[n] = love.graphics.newImage(path)
		end
		return self.descriptionTextImageList[n]
	end
end

function Character:getSound(num) --NUMBER
	if(num == -1)then
		return nil
	end
	if(self.voiceList[num] == nil)then
		path = self.voiceFolder..num..".wav"
		self.voiceList[num] = love.audio.newSource(path, "static")
	end
	if(self.voiceList[num]==nil)then
		error("LoveStory: Character Sound "..num..".wav not found!")
	end
	return self.voiceList[num]
end

return Character()