Lovestory = class("Lovestory") --LET THIS BE A STATIC CLASS (only one instance)

novel = require 'novel'
require 'lovestory.character'
require 'lovestory.dialogue'
require 'lovestory.decision'
soundManager = require 'lovestory.soundManager'


function Lovestory:init()
	
	--SETTINGS/Graphical Objects
	bgmFolder = "bgm/"
	backgroundsFolder = "backgrounds/"
	font = love.graphics.newFont("lovestory/assets/SitkaSmall.ttf",32)
	love.graphics.setFont(font)
	textbox = love.graphics.newImage("lovestory/assets/textbox.png")

	--GLOBAL VARIABLES
	index = 1
	currentBG = -1
	lastSoundIndex = -1
	decisionInProgress = nil
	queue = {} --Dialogue QUEUE
	bgList = {}
	bgMusicList = {}
	joycons = nil
	currentBGM = -1
	currentBGMplaying = -1
	tvMode = false

	--FLAGS
	skipMode = false
	textBoxShown = true
	characterDescriptionScreen = false
	swipeGesture = false
	descriptionTextHidden = false
	isBusy = false

end

------ GAME PAD / INPUT ------------

function Lovestory:gamepadpressed(joystick,button)

	if(lovestory:isBusy() == false)then
		return
	end
	
	joycons = joystick

	if(decisionInProgress ~= nil)then
		decisionInProgress:gamepadpressed(joystick,button)
		return
	end
	
	if(characterDescriptionScreen)then
		if(button == "l") then
			descriptionTextHidden = true
		end
		return
	end

	if(button == "a")then
		lovestory:indexUp()
	end
	if(button == "zl") then
		lovestory:activateSkip()
	end
	if(button == "l") then
		lovestory:hideTextbox()
	end
	if(button == "r") then
		lovestory:showCharacterDescription()
	end
end

function Lovestory:gamepadreleased(joystick,button)

	if(lovestory:isBusy() == false)then
		return
	end
	
	joycons = joystick

	if(characterDescriptionScreen)then
		if(button == "l") then
			descriptionTextHidden = false
		end
	else
		lovestory:showTextbox()
	end
	if(button == "r") then
		lovestory:hideCharacterDescription()
	end
	if(button == "zl") then
		lovestory:deactivateSkip()
	end
	if(button == "l") then
		lovestory:showTextbox()
	end

end

function Lovestory:touchreleased(id,x,y,dx,dy,pressure)

	if(lovestory:isBusy() == false)then
		return
	end
	
	if(characterDescriptionScreen)then
		return
	end
	
	if(decisionInProgress ~= nil)then
		decisionInProgress:touchreleased(id,x,y,dx,dy,pressure)
	else
		if(swipeGesture == false)then
			lovestory:indexUp()
		end
	end
	
	if(swipeGesture)then
		swipeGesture = false
	end
end

function Lovestory:touchmoved(id,x,y,dx,dy,pressure)	

	if(lovestory:isBusy() == false)then
		return
	end
	
	--Char Description Screen
	if(characterDescriptionScreen)then
		if(tvMode == true)then
			return
		end
		if(x > 880)then
			if(dx > 10 )then
				descriptionTextHidden = true
			end
		end
		if(x > 1100)then
			if(dx < -10 )then
				descriptionTextHidden = false
			end
		end
		return
	end

	--Normal Screen
	if(y > 477 and decisionInProgress == nil)then
		 if(dy > 10 )then
			self:hideTextbox()
			swipeGesture = true
		 end
	end
	if(y > 650 and decisionInProgress == nil)then
		  if(dy < -10 )then
			self:showTextbox()
			swipeGesture = true
		 end
	end
end		        

function Lovestory:gamepadaxis(joystick, axis, value, threshhold)

	if(decisionInProgress ~= nil)then
		decisionInProgress:gamepadaxis(joystick, axis, value, threshhold)
		return
	end
end

function Lovestory:hideCharacterDescription()
	characterDescriptionScreen = false
end
function Lovestory:showCharacterDescription()
	characterDescriptionScreen = true
	descriptionTextHidden = false
end

function Lovestory:hideTextbox()
	textBoxShown = false
end
function Lovestory:showTextbox()
	textBoxShown = true
end

function Lovestory:activateSkip()
	skipMode = true
end
function Lovestory:deactivateSkip()
	skipMode = false
end

function Lovestory:isBusy()
	return isBusy
end

function Lovestory:endDialogue()
	isBusy = false
end

function Lovestory:indexUp()
	if(decisionInProgress ~= nil)then
		return
	end
	
	if(textBoxShown == false)then
		self:showTextbox()
	end
	
	index = index + 1
end

function Lovestory:setBackgroundFolder(path)
	backgroundsFolder = path
end
function Lovestory:setBGMFolder(path)
	bgmFolder = path
end

function Lovestory:showBG(num)
	currentBG = num
end
function Lovestory:hideBG()
	currentBG = -1
end

function Lovestory:changeBGM(num)
	
	if(bgMusicList[num] == nil)then
			path = bgmFolder..num..".ogg"
			bgMusicList[num] = love.audio.newSource(path, "stream")
	end
	bgMusicList[num]:setLooping(true)
	currentBGM = num
end

--DIALOGUE FUNCTIONS--

function Lovestory:addDialogue(character,text)
	queue[#queue+1] = Dialogue("dialogue",character,text,-1,nil,currentBG) --text should be max 180 char long
	isBusy = true
end

function Lovestory:addComplexDialogue(character,text,voice,secondCharacter) --text should be max 180 char long
	queue[#queue+1] = Dialogue("dialogue",character,text,voice,secondCharacter,currentBG)
	isBusy = true
end

function Lovestory:addEmptyDialogue()
	queue[#queue+1] = Dialogue("empty",nil,nil,nil,nil,currentBG)
	isBusy = true
end

function Lovestory:addDecision(d)
	queue[#queue+1] = Dialogue("decision",d,nil,nil,nil,currentBG)
	isBusy = true
end

function Lovestory:bgmChange(num)
	queue[#queue+1] = Dialogue("bgmChange",num,nil,nil,nil,currentBG)
end

-------------
function Lovestory:update()
	if(decisionInProgress ~= nil)then
		if(decisionInProgress.answer > 0)then
			decisionInProgress:callback()
			decisionInProgress = nil
			self:indexUp()
		end
	end
end

function Lovestory:draw()

	if(queue[index] == nil)then

		return
	end
	
	--Load Dialogue
	dialogue = queue[index]
	
	--Music
	if(currentBGM == -1)then
		--soundManager:stop()
	else
		if(currentBGMplaying ~= currentBGM)then
			if(bgMusicList[currentBGM] ~= null)then
				soundManager:playMusic(bgMusicList[currentBGM])
				currentBGMplaying = currentBGM
			end
		end
	end
	
	--Background Image
	if(dialogue.bgNum ~= -1)then
		if(bgList[dialogue.bgNum] == nil)then
			bgPath = backgroundsFolder..dialogue.bgNum..".png"
			bgList[dialogue.bgNum] = love.graphics.newImage(bgPath)
		end
		love.graphics.draw(bgList[dialogue.bgNum],0,0)
	end
	
	--Decision Case
	if(dialogue.typ == "decision")then
		decision = dialogue.decision
		decisionInProgress = decision
		decision:draw()
		return
	end
	
	if(dialogue.typ == "bgmChange")then
		self:changeBGM(-1)
		self:changeBGM(dialogue.bgmNum)
		self:indexUp()
		return
	end
	
	if(dialogue.typ == "empty")then
		return --EMPTY FRAME => Print nothing but the Background for one Dialogue-Frame
	end
	
	--Show Character Description Screen
	if(characterDescriptionScreen)then
		descriptionImage = dialogue.character:getDescriptionImage(dialogue.currentDescriptionImage)
		descriptionText = dialogue.character:getDescriptionTextImage()
		if(descriptionImage ~= nil)then
			if(tvMode == false)then
				love.graphics.draw(descriptionImage,0,0)
				if(descriptionText ~= nil and descriptionTextHidden == false)then
					love.graphics.draw(descriptionText,0,0)
				end
				return
			elseif(tvMode == true)then
				love.graphics.draw(descriptionImage, 1290, 0, 1.5708, 0.6, 0.6)
				if(descriptionText ~= nil and descriptionTextHidden == false)then
					love.graphics.draw(descriptionText, 750, -600, 1.5708, 1, 1)
				end
				return
			end
		else
			characterDescriptionScreen = false
		end
	end
	
	--CharacterVoice
	if(dialogue.voice ~= -1)then
		if(lastSoundIndex ~= index)then
			voiceSource = dialogue.character:getSound(dialogue.voice)
			if(voiceSource ~= nil)then
				soundManager:playSound(voiceSource)
			end
			lastSoundIndex = index
		end
	end
	
	--CharacterImage
	currentStyleImage = dialogue.character:getStyleImage(dialogue.currentStyle)
	if(currentStyleImage ~= nil)then
		love.graphics.draw(currentStyleImage,640,35)
	end
	
	--SecondCharacterImage
	if(dialogue.secondCharacter ~= nil)then
		secondCharacterStyle = dialogue.secondCharacter:getStyleImage(dialogue.secondCharacterStyle)
		if(secondCharacterStyle ~= nil)then
			love.graphics.setColor(1,1,1,0.8)
			love.graphics.draw(secondCharacterStyle, 640, 35, 0, -1, 1)
			love.graphics.setColor(1,1,1,1)
		end
	end
	
	--ShowTextbox + Text
	if(textBoxShown)then
		love.graphics.draw(textbox, 0, 477)
		love.graphics.print((dialogue.character.name..":"), 20, 500)
		love.graphics.print(dialogue.text, 20, 540)
	end
	
	--SkipMode
	if(skipMode)then
		lovestory:indexUp()
	end
end

return Lovestory()